import os
import json
import stripe
import logging
import datetime
from pydantic import BaseModel
import azure.cosmos.exceptions as exceptions

from fastapi import APIRouter, Header, Request
from fastapi.responses import RedirectResponse
from fastapi import HTTPException, status, Form

from ..database.mycosmosdb import callConnect
from ..configs.config import stripe_webhook



class CReqs(BaseModel):
    oid: str
    city: str
    email: str
    session_id: str
    plan: str

    

checkoutroute = APIRouter()

NODE_DEV = os.getenv("NODE_DEV")

def read_item(container, doc_id, account_number):
    response = container.read_item(item=doc_id, partition_key=account_number)
    return response


if NODE_DEV == "production":
    MY_DOMAIN = "https://metricplatformapi.com/"
    MY_HOME = "https://www.metric-platform.com/"
elif NODE_DEV == "test":
    MY_DOMAIN = "https://metricplatform-staging.azurewebsites.net/"
    MY_HOME = "https://metricplatform-info-metric-esgc-metric-dev.vercel.app/"
else:
    MY_DOMAIN = "http://localhost:8000/"
    MY_HOME = "http://localhost:3000/"



"""
Checkout to purchase a subscription
payment_method_collection="if_required", for free trials in future
"""
@checkoutroute.post("/create-checkout-session")
async def create_checkout_session(lookup_key: str = Form(...)):
    try:
        checkout_session = stripe.checkout.Session.create(
            success_url=MY_HOME+"signup/?success=true&session_id={CHECKOUT_SESSION_ID}",
            cancel_url=MY_HOME+"signup",
            payment_method_types=["card"],
            allow_promotion_codes=True,
            payment_method_collection="if_required",
            mode="subscription",
            line_items=[
                {
                    "price": lookup_key,
                    "quantity": 1
                }
            ],
        )
        return RedirectResponse(checkout_session.url, status.HTTP_303_SEE_OTHER)

    except Exception as e:
        raise HTTPException(403, str(e))    



"""
Checkout to upgrade subscription from dataanalytics
"""
@checkoutroute.post("/create-checkout-sessionCHART")
async def create_checkout_session(lookup_key: str = Form(...)):
    try:
        checkout_session = stripe.checkout.Session.create(
            success_url=MY_HOME+"dashboard/dataanalytics/?success=true&session_id={CHECKOUT_SESSION_ID}",
            cancel_url=MY_HOME+"dashboard/dataanalytics",
            payment_method_types=["card"],
            allow_promotion_codes=True,
            payment_method_collection="if_required",
            mode="subscription",
            line_items=[
                {
                    "price": lookup_key,
                    "quantity": 1}
                ],
        )
        return RedirectResponse(checkout_session.url, status.HTTP_303_SEE_OTHER)
    except Exception as e:
        raise HTTPException(403, str(e))


"""
Checkout to upgrade subscription from dataanalytics
"""
@checkoutroute.post("/create-checkout-sessionIS")
async def create_checkout_session(lookup_key: str = Form(...)):
    try:
        checkout_session = stripe.checkout.Session.create(
            success_url=MY_HOME+"dashboard/improvementstrategies/?success=true&session_id={CHECKOUT_SESSION_ID}",
            cancel_url=MY_HOME+"dashboard/improvementstrategies",
            payment_method_types=["card"],
            allow_promotion_codes=True,
            payment_method_collection="if_required",
            mode="subscription",
            line_items=[
                {
                    "price": lookup_key,
                    "quantity": 1}
                ],    
        )
        return RedirectResponse(checkout_session.url, status.HTTP_303_SEE_OTHER)

    except Exception as e:
        raise HTTPException(403, str(e))

"""
Checkout for 
"""
@checkoutroute.post("/create-checkout-session-2")
async def create_checkout_session(lookup_key: str = Form(...), number: int = Form(...)):
    try:
        checkout_session = stripe.checkout.Session.create(
            success_url=MY_HOME+"dashboard/fundoverview/?success=true&session_id={CHECKOUT_SESSION_ID}",
            cancel_url=MY_HOME+"dashboard/fundoverview",
            payment_method_types=["card"],
            mode="subscription",
            allow_promotion_codes=True,
            payment_method_collection="if_required",
            line_items=[
                {
                    "price": lookup_key,
                    "quantity": number}
                ],
            )
        return RedirectResponse(checkout_session.url, status.HTTP_303_SEE_OTHER)

    except Exception as e:
        raise HTTPException(403, str(e))
    

"""
Checkout for 
"""
@checkoutroute.post("/create-checkout-session-3")
async def create_checkout_session(lookup_key: str = Form(...), number: int = Form(...)):
    try:
        checkout_session = stripe.checkout.Session.create(
            success_url=MY_HOME+"signup/?success=true&session_id={CHECKOUT_SESSION_ID}",
            cancel_url=MY_HOME+"signup",
            payment_method_types=["card"],
            mode="subscription",
            allow_promotion_codes=True,
            payment_method_collection="if_required",
            line_items=[
                {
                    "price": lookup_key,
                    "quantity": number}
                ],
            )
        return RedirectResponse(checkout_session.url, status.HTTP_303_SEE_OTHER)

    except Exception as e:
        raise HTTPException(403, str(e))



"""
Purchase carbon offsets
"""
@checkoutroute.post("/create-checkout-session-4")
async def create_checkout_session(lookup_key: str = Form(...), number: int = Form(...)):
    try:
        checkout_session = stripe.checkout.Session.create(
            success_url=MY_HOME+"dashboard/carbonoffsets/?success=true&session_id={CHECKOUT_SESSION_ID}",
            cancel_url=MY_HOME+"dashboard/carbonoffsets",
            payment_method_types=["card"],
            mode="payment",
            line_items=[
                {
                    "price": lookup_key,
                    "quantity": number}
                ],
            )
        return RedirectResponse(checkout_session.url, status.HTTP_303_SEE_OTHER)

    except Exception as e:
        raise HTTPException(403, str(e))

    
"""
For demonstration purposes, we're using the Checkout session to retrieve the customer ID.
Typically this is stored alongside the authenticated user in your database.
"""
@checkoutroute.post('/create-portal-session')
async def customer_portal(session_id: str = Form(...)):
    try:
        checkout_session = stripe.checkout.Session.retrieve(session_id)
        return_url = MY_HOME
        portalSession = stripe.billing_portal.Session.create(customer=checkout_session.customer, return_url=return_url)
        return RedirectResponse(portalSession.url, status.HTTP_303_SEE_OTHER)

    except Exception as e:
        raise HTTPException(403, str(e))



"""
Stripe API Webhook Endpoint, send results to the Database
"""
@checkoutroute.post("/webhook")
async def webhook_received(request: Request, stripe_signature: str = Header(None)):
    webhook_secret = stripe_webhook
    data = await request.body()
    deserial = json.loads(data)
    nowZ = datetime.datetime.now()
    yearZ = nowZ.strftime("%Y")
    monthZ = nowZ.strftime("%m")
    dayZ = nowZ.strftime("%d")
    hourZ = nowZ.strftime("%H")
    minuteZ = nowZ.strftime("%M")
    all_timeZ = yearZ+"-"+monthZ+"-"+dayZ+"T"+hourZ+":"+minuteZ

    try:
        event = stripe.Webhook.construct_event(
            payload=data,
            sig_header=stripe_signature,
            secret=webhook_secret
        )
    except Exception as e:
        return {"error": str(e)}

    event_type = event['type']
    if event_type == 'customer.subscription.updated':
        cc = deserial["data"]["object"]["cancel_at_period_end"]
        if cc:
            # if cancelled then do nothing for now + reconnect to db if checkout session expires
            sub_id = deserial["data"]["object"]["id"]
            plan_id = deserial["data"]["object"]["items"]["data"][0]["plan"]["id"]
            try:
                db_result = read_item(callConnect.container14, sub_id, sub_id)
                userx = db_result
                meta = db_result["meta"]
                meta.append({
                    "plan_id": plan_id,
                    "start_date": all_timeZ,
                    "type": "canceled"
                })
                metaN = {
                   "meta": meta
                }
                userx.update(metaN)
                callConnect.container14.upsert_item(body=userx)
                try:
                    user_record = read_item(callConnect.container, db_result["oid"], db_result["city"])
                    userr = user_record
                    planN = {
                        "plan_id": plan_id
                    }
                    userr.update(planN)
                    callConnect.container.upsert_item(body=userr)
                except:
                    logging.error('This is an error message for /webhook we do not have this user record')
                    pass
            except exceptions.CosmosHttpResponseError as e:
                logging.error('This is an error message for /webhook we do not have this sub_id')
                pass
            pass
        else:
            sub_id = deserial["data"]["object"]["id"]
            plan_id = deserial["data"]["object"]["items"]["data"][0]["plan"]["id"]
            try:
                db_result = read_item(callConnect.container14, sub_id, sub_id)
                userx = db_result
                meta = db_result["meta"]
                prior_plan = meta[-1]["plan_id"]
                if prior_plan == "price_1KUEG2JnvGQfuwqX5krVeoWW" and plan_id == "price_1LrjAwJnvGQfuwqXXIkFAbyl":
                    meta.append({
                        "plan_id": plan_id,
                        "start_date": all_timeZ,
                        "type": "upgrade"
                    })
                elif prior_plan == "price_1LrjAwJnvGQfuwqXXIkFAbyl" and plan_id == "price_1KUEG2JnvGQfuwqX5krVeoWW":
                    meta.append({
                        "plan_id": plan_id,
                        "start_date": all_timeZ,
                        "type": "downgrade"
                    })
                else:
                    meta.append({
                        "plan_id": plan_id,
                        "start_date": all_timeZ,
                        "type": "unknown"
                    })
                metaN = {
                   "meta": meta
                }
                userx.update(metaN)
                callConnect.container14.upsert_item(body=userx)
                try:
                    user_record = read_item(callConnect.container, db_result["oid"], db_result["city"])
                    userr = user_record
                    planN = {
                        "plan_id": plan_id
                    }
                    userr.update(planN)
                    callConnect.container.upsert_item(body=userr)
                except:
                    logging.error('This is an error message for /webhook we do not have this user record')
                    pass

            except exceptions.CosmosHttpResponseError as e:
                logging.error('This is an error message for /webhook we do not have this sub_id')
                pass
            pass
    else:
        pass



"""
Retreive Checkout Session ID
"""
@checkoutroute.post('/retreive-checkoutsession')
async def retreive_checkout(chk: CReqs):
    nowZ = datetime.datetime.now()
    yearZ = nowZ.strftime("%Y")
    monthZ = nowZ.strftime("%m")
    dayZ = nowZ.strftime("%d")
    hourZ = nowZ.strftime("%H")
    minuteZ = nowZ.strftime("%M")
    all_timeZ = yearZ+"-"+monthZ+"-"+dayZ+"T"+hourZ+":"+minuteZ
    
    try:
        checkout_session = stripe.checkout.Session.retrieve(chk.session_id)
        sub_id = checkout_session["customer"]
        try:
            userx = {
                "id": sub_id,
                "oid": chk.oid,
                "city": chk.city,
                "email": chk.email,
                "meta": [
                    {
                        "plan_id": chk.plan,
                        "start_date": all_timeZ,
                        "type": "origin"
                    }
                ]
            }
            callConnect.container14.upsert_item(body=userx)
        except:
            pass

    except Exception as e:
        raise HTTPException(403, str(e))

